// -*- C++ -*-
//
//  This file is part of Siena, a wide-area event notification system.
//  See http://www.cs.colorado.edu/serl/siena/
//
//  Authors: See the file AUTHORS for full details.
//
//  Copyright (C) 2002-2004 University of Colorado
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
//  USA, or send email to serl@cs.colorado.edu.
//
// $Id: ssimp_types.cc,v 1.4 2008-08-21 15:34:11 carzanig Exp $
//
#include "siena/ssimp_types.h"

using namespace std;
using namespace siena;

namespace ssimp {

ostream& Constraint :: sff(ostream& out) const
{
  out << m_name;
  switch( m_op )
    {
    case eq_id: out << " = "; break;
    case lt_id: out << " < "; break;
    case gt_id: out << " > "; break;
    case sf_id: out << " *= "; break;
    case pf_id: out << " =* "; break;
    case ss_id: out << " ** "; break;
    case ne_id: out << " != "; break;
    default: out << "??"; break;
    }
  switch( m_value.m_type )
    {
    case bool_id: out << (m_value.m_bool ? "true" : "false"); break;
    case double_id: out << m_value.m_double; break;
    case int_id: out << m_value.m_int; break;
    case string_id: out << "\"" << m_value.m_string << "\""; break;
    default: out << "???"; break;
    }
  return out;
}

string Constraint :: str() const
{
  ostringstream oss;
  oss << "[Constraint: name=" << m_name
      << ",operator=";
  switch( m_op )
    {
    case eq_id: oss << "="; break;
    case lt_id: oss << "<"; break;
    case gt_id: oss << ">"; break;
    case sf_id: oss << "*="; break;
    case pf_id: oss << "=*"; break;
    case ss_id: oss << "**"; break;
    case ne_id: oss << "!="; break;
    default: oss << "??"; break;
    }

  oss << ",type=";

  switch( m_value.m_type )
    {
    case bool_id: oss << "bool"; break;
    case double_id: oss << "double"; break;
    case int_id: oss << "int"; break;
    case string_id: oss << "string"; break;
    default: oss << "??"; break;
    }
	
  oss << ",value=";

  switch( m_value.m_type )
    {
    case bool_id: oss << m_value.m_bool; break;
    case double_id: oss << m_value.m_double; break;
    case int_id: oss << m_value.m_int; break;
    case string_id: oss << "\"" << m_value.m_string << "\""; break;
    default: oss << "???"; break;
    }
  oss << "]";
  return oss.str();
}

size_t Message :: Value :: bytesize() const
{
  size_t size = sizeof( type );
  switch( type )
    {
    case bool_id:
      size += sizeof( b_value );
      break;
    case double_id:
      size += sizeof( d_value );
      break;
    case int_id:
      size += sizeof( i_value );
      break;
    default:
      size += s_value.size();
    }
  return size;
}

string Message :: Value :: str() const
{
  ostringstream oss;
  oss << "[Message::Value: type=" << type;
  switch( type )
    {
    case bool_id:
      oss << ",b_value=" << b_value;
      break;
    case double_id:
      oss << ",d_value=" << d_value;
      break;
    case int_id:
      oss << ",i_value=" << i_value;
      break;
    default:
      oss << ",s_value=" << s_value;
    }
  oss << "]";
  return oss.str();
}

size_t Message :: bytesize() const
{
  size_t size = 0;
  map<string,Value>::const_iterator i;
  for( i = m_attrMap.begin(); i != m_attrMap.end(); ++i )
    {
      size += (*i).first.length() + (*i).second.bytesize();
    }
  return size;
}

string Message :: str() const
{
  std::ostringstream oss;
  oss << "[Message:";

  map<string,Value>::const_iterator i;
  for( i=m_attrMap.begin(); i!=m_attrMap.end(); i++ )
    {
      oss << "," << (*i).first << "->" << (*i).second.str();
    }

  oss << "]";
  return oss.str();
}

} // namespace ssimp
