// -*- C++ -*-
//
//  This file is part of Siena, a wide-area event notification system.
//  See http://www.cs.colorado.edu/serl/siena/
//
//  Authors: See the file AUTHORS for full details.
//
//  Copyright (C) 2002-2004 University of Colorado
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
//  USA, or send email to serl@cs.colorado.edu.
//
// $Id: covering.cc,v 1.3 2008-08-21 15:34:10 carzanig Exp $
//
#include <cassert>
#include <iostream>
#include <vector>
#include "siena/simplify.h"
#include "siena/ssimp_types.h"

using namespace siena;
using namespace std;

int main( int argc, char** argv )
{
  {
    Constraint c1( string( "x" ), eq_id, (int_t)10 );
    Constraint c1a( c1 );
    Constraint c2( string( "x" ), lt_id, (int_t)11 );
    Constraint c3( string( "x" ), lt_id, (int_t)14 );
    Constraint c4( string( "x" ), gt_id, (int_t)8 );
    Constraint c5( string( "x" ), gt_id, (int_t)5 );
    Constraint c6( string( "x" ), sf_id, string( "ing" ) );
    Constraint c7( string( "x" ), pf_id, string( "jump" ) );
    Constraint c8( string( "x" ), ss_id, string( "pi" ) );
    Constraint c9( string( "x" ), eq_id, string( "jumping" ) );
    Constraint c10( string( "x" ) );
        
    assert( covers( c3, c2 ) ); // x < 14 covers x < 11
    assert( covers( c2, c1 ) ); // x < 11 covers x = 10
    assert( !covers( c2, c3 ) );
    assert( !covers( c1, c2 ) );
    
    assert( covers( c5, c4 ) ); // x > 5 covers x > 8
    assert( !covers( c4, c5 ) );
    assert( covers( c4, c1 ) ); // x > 8 covers x = 10
    assert( !covers( c1, c4 ) );
    assert( covers( c5, c1 ) );// x > 5 covers x = 10
    assert( !covers( c1, c5 ) );
    
    assert( c1 == c1a );
    assert( !(c1 == c2) );

    assert( covers( c6, c9 ) );
    assert( covers( c7, c9 ) );
    assert( covers( c8, c9 ) );

    assert( !covers( c9, c6 ) );
    assert( !covers( c9, c7 ) );
    assert( !covers( c9, c8 ) );

    // anytype covers all, not covered
    assert( covers( c10, c1 ) );
    assert( covers( c10, c2 ) );
    assert( covers( c10, c3 ) );
    assert( covers( c10, c4 ) );
    assert( covers( c10, c5 ) );
    assert( covers( c10, c6 ) );
    assert( covers( c10, c7 ) );
    assert( covers( c10, c8 ) );
    assert( covers( c10, c9 ) );

    assert( !covers( c1, c10 ) );
    assert( !covers( c2, c10 ) );
    assert( !covers( c3, c10 ) );
    assert( !covers( c4, c10 ) );
    assert( !covers( c5, c10 ) );
    assert( !covers( c6, c10 ) );
    assert( !covers( c7, c10 ) );
    assert( !covers( c8, c10 ) );
    assert( !covers( c9, c10 ) );
  }
   
  { 
    // constraints are unrelated (different names)
    Constraint c1( string( "a" ), eq_id, (int_t)10 );
    Constraint c2( string( "b" ), lt_id, (int_t)11 );
    Constraint c3( string( "c" ), lt_id, (int_t)14 );
    Constraint c4( string( "d" ), gt_id, (int_t)8 );
    Constraint c5( string( "e" ), gt_id, (int_t)5 );

    BasicFilter f1;
    f1.add( c2 );
    f1.add( c1 );
    f1.add( c4 );
    f1.add( c5 );
    
    BasicFilter f2;
    f2.add( c2 );
    f2.add( c4 );
    
    BasicFilter f3;
    f3.add( c1 );
    f3.add( c5 );
    
    assert( !covers( f1, f2 ) );
    assert( covers( f2, f1 ) );
    
    assert( !covers( f1, f3 ) );
    assert( covers( f3, f1 ) );
    
    assert( !covers( f2, f3 ) );
    assert( !covers( f3, f2 ) );
    
    BasicPredicate p1;
    p1.add( f1 );
    
    BasicPredicate p2;
    p2.add( f2 );
    p2.add( f3 );
    
    assert( covers( p2, p1 ) );
    assert( !covers( p1, p2 ) );
  }
  return 0;
}
